import java.awt.Shape;
import java.awt.geom.AffineTransform;


/**
 * Abstrakte Klasse Moebel
 * @author  claus albowski
 */

public abstract class Moebel
{
  protected int xPosition;
  protected int yPosition;
  protected int orientierung;
  protected String farbe;
  protected boolean istSichtbar;
  protected int breite;
  protected int tiefe;


  /**
   * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
   */
  protected abstract Shape gibAktuelleFigur();
  
  /**
   * Transformiert das Shape, so dass es die richtige Position
   * und Orientierung einnimmt.
   */
  protected Shape transformiere(Shape shape)
  {
      AffineTransform t = new AffineTransform();
      t.rotate(Math.toRadians(orientierung),gibMitteX(),gibMitteY());
      t.translate(xPosition, yPosition);
      return  t.createTransformedShape(shape);
  }
  
  /**
   * Die Methode transformiere(Shape) knnte berschrieben werden,
   * hier wird sie durch berladen ersetzt. Notwendig ist es,
   * um die Verzweigung fr dieTransformation genau dahin zu geben,
   * wohin sie gehrt. (--> Kohsion)
   */
  protected Shape transformiere(Shape shape, AffineTransform beispielTransformation)
  {
      AffineTransform t = new AffineTransform();
      t.rotate(Math.toRadians(orientierung),gibMitteX(),gibMitteY());
      t.translate(xPosition, yPosition);
      t.concatenate(beispielTransformation);
      return  t.createTransformedShape(shape);
  }
  
  /**
   * Hole die X-Koordinate des Mittelpunktes
   * [ Hilfsfunktion fr das Drehen. Kann bei einfachen Objekten private sein.]
   */
  protected int gibMitteX()
  {
    return xPosition+breite/2;
  }

  /**
   * Hole die Y-Koordinate des Mittelpunktes
   * [ Hilfsfunktion fr das Drehen. Kann bei einfachen Objekten private sein.]
   */
  protected int gibMitteY()
  {
    return yPosition+tiefe/2;
  }

  /**
   * Mache dieses Objekt sichtbar. Wenn es bereits sichtbar ist, tue
   * nichts.
   */
  public void zeige()
  {
    istSichtbar = true;
    zeichne();
  }

  /**
   * Mache dieses Objekt unsichtbar. Wenn es bereits unsichtbar ist, tue
   * nichts.
   */
  public void verberge()
  {
    loesche();
    istSichtbar = false;
  }

  /**
   * Drehe auf den angegebenen Winkel
   */
  public void dreheAuf(int neuerWinkel)
  {
     loesche();
     orientierung = neuerWinkel;
     zeichne();
  }

  /**
   * Bewege dieses Objekt horizontal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeHorizontal(int entfernung)
  {
    loesche();
    xPosition += entfernung;
    zeichne();
  }

  /**
   * Bewege dieses objekt vertikal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeVertikal(int entfernung)
  {
    loesche();
    yPosition += entfernung;
    zeichne();
  }

 
  /**
   * ndere die Farbe dieses Objektes in 'neueFarbe'.
   * Gltige Angaben sind "rot", "gelb", "blau", "gruen",
   * "lila" und "schwarz".
   */
  public void aendereFarbe(String neueFarbe)
  {
    farbe = neueFarbe;
    zeichne();
  }

  /*
   * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
   */
  protected void zeichne()
  {
    if (istSichtbar)
    {
      Shape figur = gibAktuelleFigur();
      Leinwand leinwand = Leinwand.gibLeinwand();
      leinwand.zeichne (
        this,
        farbe,
        figur);
      leinwand.warte(10);
    }
  }

  /*
   * Lsche dieses Objekt vom Bildschirm.
   */
  protected void loesche()
  {
    if (istSichtbar)
    {
      Leinwand leinwand = Leinwand.gibLeinwand();
      leinwand.entferne(this);
    }
  }
}
